using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using Infragistics.Windows.Ribbon;
using Infragistics.Windows.Ribbon.Events;
using System.IO;
using Infragistics.Windows.Helpers;


namespace Ribbon_With_RichTextBox
{
    /// <summary>
    /// Interaction logic for MyRibbonWindow.xaml
    /// </summary>
    public partial class MyRibbonWindow : XamRibbonWindow
    {
        #region Members

        private bool isUpdatingFontInfo;
        private bool isSelectionValid;
        private MemoryStream oldTextInfo;

        #endregion //Members

        #region Constructor

        /// <summary>
        /// Initializes a new instance the class.
        /// </summary>
        public MyRibbonWindow()
        {            
            InitializeComponent();
        }
        #endregion //Constructor        

        #region Properties

        #region IsDocumentOpen

        /// <summary>
        /// Identifies the "IsDocumentOpen" property.
        /// </summary>
        public static readonly DependencyProperty IsDocumentOpenProperty =
            DependencyProperty.Register("IsDocumentOpen",
            typeof(bool), typeof(MyRibbonWindow),
            new FrameworkPropertyMetadata(KnownBoxes.TrueBox));

        /// <summary>
        /// Gets or sets whether there is currently a document editing surface shown.
        /// </summary>
        public bool IsDocumentOpen
        {
            get
            {
                return (bool)this.GetValue(IsDocumentOpenProperty);
            }
            set
            {
                this.SetValue(IsDocumentOpenProperty, value);
            }
        }
        #endregion //IsDocumentOpen

        #region IsSelectionValid

        /// <summary>
        /// Gets whether the any text is currently selected in the associated RichTextBox 
        /// </summary>
        public bool IsSelectionValid
        {
            get
            {
                return this.isSelectionValid;
            }
            set
            {
                if (this.isSelectionValid != value)
                {
                    this.isSelectionValid = value;
                    this.myRibbon.ContextualTabGroups[0].IsVisible = this.isSelectionValid;
                }
            }
        }
        #endregion //IsSelectionValid

        #endregion //Properties

        #region Event Handlers

        #region OnClipArtButtonClick

        private void OnClipArtButtonClick(object sender, RoutedEventArgs e)
        {
            ButtonTool tool = e.Source as ButtonTool;
            if (tool != null)
            {
                Image image = new Image();
                image.Width = tool.LargeImage.Width;
                image.Source = tool.LargeImage;

                Paragraph p = new Paragraph(new InlineUIContainer(image, this.richTextBox.Selection.Start));
                this.richTextBox.Document.Blocks.Add(p);
            }
        }
        #endregion //OnClipArtButtonClick

        #region OnCloseButtonClick

        private void OnCloseButtonClick(object sender, RoutedEventArgs e)
        {
            this.SaveContent(false);
            this.richTextBox.Document.Blocks.Clear();
            this.richTextBox.Visibility = Visibility.Hidden;            
        }
        #endregion //OnCloseButtonClick

        #region OnColorPicked

        private void OnColorPicked(object sender, GalleryItemEventArgs e)
        {
            if (e.Item == null || e.Item.Tag == null)
                return;

            Color newColor = (Color)e.Item.Tag;
            SolidColorBrush newBrush = new SolidColorBrush(newColor);
            if (e.GalleryTool.Id == "globalColorPicker")
            {                
                this.richTextBox.SetValue(RichTextBox.ForegroundProperty, newBrush);
            }
            else if (e.GalleryTool.Id == "selectionColorPicker")
            {
                this.richTextBox.Selection.ApplyPropertyValue(RichTextBox.ForegroundProperty, newBrush);
            }
        }
        #endregion //OnColorPicked

        #region OnFontInfoChanged

        private void OnFontInfoChanged(object sender, RoutedPropertyChangedEventArgs<object> e)
        {
            if (this.isUpdatingFontInfo)
                return;

            if (e.Source == this.fontFamilyCombo_Selection)
            {
                FontFamily fontFamily = (FontFamily)this.fontFamilyCombo_Selection.SelectedItem;
                this.richTextBox.Selection.ApplyPropertyValue(TextElement.FontFamilyProperty, fontFamily);
            }
            else if (e.Source == this.fontSizeCombo_Selection && this.fontSizeCombo_Selection.Value != null)
            {
                double fontSize = (double)this.fontSizeCombo_Selection.SelectedItem;
                this.richTextBox.Selection.ApplyPropertyValue(TextElement.FontSizeProperty, fontSize);
            }
        }
        #endregion //OnFontInfoChanged

        #region OnGalleryItemActivated

        // We want to set either the selected text or the entire document's style to whatever
        // the associated style of the gallery item is, which is the Live Preview feature in Office 2007
        private void OnGalleryItemActivated(object sender, GalleryItemEventArgs e)
        {       
            FontFamily newfontFamily;
            double newFontSize;
            SolidColorBrush newForeBrush;
            string newFontStyle;

            if (e.Item != null && e.Item.Tag != null)
            {
                // Parse out the style information from the tag and apply it to the selection/document
                this.ParseGalleryItemTag(e.Item.Tag.ToString(), out newfontFamily, out newFontSize, out newForeBrush, out newFontStyle);
                this.ApplyStyleToEditor(newfontFamily, newFontSize, newForeBrush, newFontStyle);
            }
            else
            {
                // Reset the selection/document to the previous stored style
                if (this.oldTextInfo != null)
                {
                    if (this.IsSelectionValid)
                        this.richTextBox.Selection.Load(this.oldTextInfo, DataFormats.XamlPackage);
                    else
                    {
                        TextRange range = new TextRange(this.richTextBox.Document.ContentStart, this.richTextBox.Document.ContentEnd);
                        range.Load(this.oldTextInfo, DataFormats.XamlPackage);                        
                    }
                }
            }
        }
        #endregion //OnGalleryItemActivated

        #region OnGalleryItemSelected

        private void OnGalleryItemSelected(object sender, GalleryItemEventArgs e)
        {
            FontFamily fontFamily;
            double fontSize;
            SolidColorBrush foreBrush;
            string fontStyle;

            this.ParseGalleryItemTag(e.Item.Tag.ToString(), out fontFamily, out fontSize, out foreBrush, out fontStyle);
            this.ApplyStyleToEditor(fontFamily, fontSize, foreBrush, fontStyle);
        }
        #endregion //OnGalleryItemSelected

        #region OnGalleryMenuOpened

        // We want to initialize the styling information for the live preview here so that
        // when a user activates a GalleryItem and then moves away from the area (or the 
        // menu is closed without selecting something), we can revert the text to the 
        // previous value
        private void OnGalleryMenuOpened(object sender, RoutedEventArgs e)
        {
            if (this.oldTextInfo != null)            
                this.oldTextInfo.Close();
            
            this.oldTextInfo = new MemoryStream(); 
            if (this.IsSelectionValid)     
                this.richTextBox.Selection.Save(this.oldTextInfo, DataFormats.XamlPackage);            
            else
            {
                TextRange range = new TextRange(this.richTextBox.Document.ContentStart, this.richTextBox.Document.ContentEnd);
                range.Save(this.oldTextInfo, DataFormats.XamlPackage);
            }
        }
        #endregion //OnGalleryMenuOpened

        #region OnNewButtonClick

        private void OnNewButtonClick(object sender, RoutedEventArgs e)
        {
            this.CreateNewDocument();            
        }
        #endregion //OnNewButtonClick

        #region OnOpenButtonClick

        private void OnOpenButtonClick(object sender, RoutedEventArgs e)
        {
            this.SaveContent(false);

            Microsoft.Win32.OpenFileDialog dlg = new Microsoft.Win32.OpenFileDialog();
            dlg.Filter = "Xaml Package File (*.xpkg)|*.xpkg;";
            if (dlg.ShowDialog(this) == true)
            {
                FileStream stream = null;
                try
                {
                    stream = new FileStream(dlg.FileName, FileMode.Open);
                    TextRange range = new TextRange(this.richTextBox.Document.ContentStart, this.richTextBox.Document.ContentEnd);
                    range.Load(stream, DataFormats.XamlPackage);
                    this.richTextBox.Visibility = Visibility.Visible;
                }
                catch (IOException ex)
                {
                    MessageBox.Show("There was an error opening the specified file: \n\n" + ex.Message);
                }
                finally
                {
                    if (stream != null)
                        stream.Close();
                }
            }
        }
        #endregion //OnOpenButtonClick

        #region OnRecentItemClick

        private void OnRecentItemClick(object sender, RoutedEventArgs e)
        {
            ButtonTool item = e.Source as ButtonTool;
            if (item != null)
            {
                this.CreateNewDocument();                    
                this.richTextBox.Document.Blocks.Clear();
                Run r = new Run(item.Content.ToString());
                this.richTextBox.Document.Blocks.Add(new Paragraph(r));
            }
        }
        #endregion //OnRecentItemClick

        #region OnRichTextBoxSelectionChanged

        private void OnRichTextBoxSelectionChanged(object sender, RoutedEventArgs e)
        {
            TextSelection selection = this.richTextBox.Selection;
            this.boldTool.IsChecked = selection.GetPropertyValue(FontWeightProperty).ToString() == "Bold";
            this.italicTool.IsChecked = selection.GetPropertyValue(FontStyleProperty).ToString() == "Italic";
            this.underlineTool.IsChecked = selection.GetPropertyValue(Inline.TextDecorationsProperty) == TextDecorations.Underline;

            // We only want to allow selection-specific actions when there is actually something selected
            this.IsSelectionValid = selection.Start != selection.End;            

            this.SynchronizeSelectionCombos();
        }
        #endregion //OnRichTextBoxSelectionChanged

        #region OnSaveButtonClick

        private void OnSaveButtonClick(object sender, RoutedEventArgs e)
        {
            this.SaveContent(true);
        }
        #endregion //OnSaveButtonClick

        #region OnThemeSelected

        private void OnThemeSelected(object sender, RoutedEventArgs e)
        {
            RadioButtonTool tool = e.Source as RadioButtonTool;
            if (tool != null)
            {
                this.myRibbon.Theme = tool.Tag.ToString();
            }
        }
        #endregion //OnThemeSelected

        #region OnWindowLoad

        private void OnWindowLoad(object sender, RoutedEventArgs e)
        {
            this.richTextBox.Focus();
        }
        #endregion //OnWindowLoad

        #endregion //Event Handlers

        #region Private Methods

        #region ApplyStyleToEditor

        private void ApplyStyleToEditor(FontFamily fontFamily, double fontSize, SolidColorBrush foreBrush, string fontStyle)
        {
            // Check to see whether we should apply the new style only to the selected text
            if (this.IsSelectionValid)
            {
                TextPointer start = this.richTextBox.Selection.Start;
                TextPointer end = this.richTextBox.Selection.End;

                TextRange range = new TextRange(start, end);

                if(fontFamily != null)
                    range.ApplyPropertyValue(RichTextBox.FontFamilyProperty, fontFamily);

                if(fontSize > 0)
                    range.ApplyPropertyValue(RichTextBox.FontSizeProperty, fontSize);

                if(foreBrush != null)
                    range.ApplyPropertyValue(RichTextBox.ForegroundProperty, foreBrush);

                if (fontStyle == "Bold")
                    range.ApplyPropertyValue(RichTextBox.FontWeightProperty, FontWeights.Bold);
                else
                    range.ApplyPropertyValue(RichTextBox.FontWeightProperty, RichTextBox.FontWeightProperty.DefaultMetadata.DefaultValue);

                if (fontStyle == "Italic")
                    range.ApplyPropertyValue(RichTextBox.FontStyleProperty, FontStyles.Italic);
                else
                    range.ApplyPropertyValue(RichTextBox.FontStyleProperty, RichTextBox.FontStyleProperty.DefaultMetadata.DefaultValue);
            }
            else
            {
                TextRange range = new TextRange(this.richTextBox.Document.ContentStart, this.richTextBox.Document.ContentEnd);

                if (fontFamily != null)
                    range.ApplyPropertyValue(RichTextBox.FontFamilyProperty, fontFamily);

                if (fontSize > 0)
                    range.ApplyPropertyValue(RichTextBox.FontSizeProperty, fontSize);

                if (foreBrush != null)
                    range.ApplyPropertyValue(RichTextBox.ForegroundProperty, foreBrush);

                if (fontStyle == "Bold")
                    range.ApplyPropertyValue(RichTextBox.FontWeightProperty, FontWeights.Bold);
                else
                    range.ApplyPropertyValue(RichTextBox.FontWeightProperty, RichTextBox.FontWeightProperty.DefaultMetadata.DefaultValue);

                if (fontStyle == "Italic")
                    range.ApplyPropertyValue(RichTextBox.FontStyleProperty, FontStyles.Italic);
                else
                    range.ApplyPropertyValue(RichTextBox.FontStyleProperty, RichTextBox.FontStyleProperty.DefaultMetadata.DefaultValue);
            }
        }
        #endregion //ApplyStyleToEditor

        #region CreateNewDocument

        private void CreateNewDocument()
        {
            this.SaveContent(false);
            this.richTextBox.Document.Blocks.Clear();
            this.richTextBox.Visibility = Visibility.Visible;
        }
        #endregion //CreateNewDocument

        #region ParseGalleryItemTag

        private void ParseGalleryItemTag(string tag, out FontFamily fontFamily, out double fontSize, out SolidColorBrush foreBrush, out string fontStyle)
        {
            string[] styleInfo = tag.Split(';');

            // Parse the font family
            fontFamily = new FontFamily(null, styleInfo[0]);
            fontSize = double.Parse(styleInfo[1]);

            // Parse the fore-color
            string[] rgb = styleInfo[2].Split(',');
            Color fontColor = Color.FromRgb(byte.Parse(rgb[0]), byte.Parse(rgb[1]), byte.Parse(rgb[2]));
            foreBrush = new SolidColorBrush(fontColor);

            fontStyle = styleInfo[3];
        }
        #endregion //ParseGalleryItemTag

        #region SaveContent

        private void SaveContent(bool isUserTriggered)
        {
            FlowDocument document = this.richTextBox.Document;
            TextRange range = new TextRange(document.ContentStart, document.ContentEnd);

            bool isDocumentEmpty = range.Text == "\r\n" || range.Text.Length == 0;

            // Don't bother showing the dialog if we have nothing to save and the user
            // didn't specifically request the save
            if (isDocumentEmpty && isUserTriggered == false)
                return;

            // Prompt the user on whether they want to save the content if this function were triggered
            // by another action (i.e. New, Close)
            if (isUserTriggered == false )
            {                
                if (MessageBoxResult.No == MessageBox.Show(this, "There are unsaved changes.  Would you like to save?", "Save File", MessageBoxButton.YesNo))
                    return;
            }

            Microsoft.Win32.SaveFileDialog dlg = new Microsoft.Win32.SaveFileDialog();
            dlg.Filter = "Xaml Package File (*.xpkg)|*.xpkg;";
            if (dlg.ShowDialog(this) == true)
            {
                FileStream stream = null;
                try
                {                    
                    stream = new FileStream(dlg.FileName, FileMode.Create);
                    range.Save(stream, DataFormats.XamlPackage);
                }
                catch (IOException e)
                {
                    MessageBox.Show("Could not save file: \n\n" + e.Message);
                }
                finally
                {
                    if (stream != null)
                        stream.Close();
                }
            }
        }
        #endregion //SaveContent

        #region SynchronizeSelectionCombos

        private void SynchronizeSelectionCombos()
        {
            this.isUpdatingFontInfo = true;

            this.fontFamilyCombo_Selection.SelectedItem =
                this.richTextBox.Selection.GetPropertyValue(RichTextBox.FontFamilyProperty);

            this.fontSizeCombo_Selection.SelectedItem =
                this.richTextBox.Selection.GetPropertyValue(RichTextBox.FontSizeProperty);            
            
            this.isUpdatingFontInfo = false;
        }
        #endregion //SynchronizeSelectionCombos

        #endregion //Private Methods
    }
}